'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _postcss = require('postcss');

var _postcss2 = _interopRequireDefault(_postcss);

var _postcssModules = require('postcss-modules');

var _postcssModules2 = _interopRequireDefault(_postcssModules);

var _postcssPluginComposition = require('postcss-plugin-composition');

var _postcssPluginComposition2 = _interopRequireDefault(_postcssPluginComposition);

var _vueTemplateCompiler = require('vue-template-compiler');

var VueCompiler = _interopRequireWildcard(_vueTemplateCompiler);

var _buble = require('vue-template-es2015-compiler/buble');

var _sourceMap = require('source-map');

var _GenId = require('./GenId');

var _GenId2 = _interopRequireDefault(_GenId);

var _PostCSSScope = require('./PostCSSScope');

var _PostCSSScope2 = _interopRequireDefault(_PostCSSScope);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Default transform options
 */

const DefaultOptions = {

  runtime: 'require("vue-compiler-runtime")',

  includeFileName: false,

  postcss: [],

  extractStyles: false,

  sourceMap: false,

  sourceMapRoot: 'vue:///',

  styleSourceMap: false,

  postcssModules: null,

  compilerOptions: null,

  serverRendering: false,

  hotReload: false,

  compilers: {},

  getCompiler: () => {}

  /**
   * Compile file into component
   *
   * @param {string}          filePath
   * @param {string}          content
   * @param {CompilerOptions} options_
   * @return {BuildResult | ErrorResult} Compile result
   */
};
exports.default = async function Compile(filePath, content, options_ = {}) {
  const options = _extends({}, DefaultOptions, options_);
  const components = VueCompiler.parseComponent(content, { outputSourceRange: true });

  const promises = [null, null, null];

  if (components.script) {
    setSourceInfo(components.script, filePath, content);
    promises[0] = processItem(components.script, options);
  }

  if (components.template) {
    setSourceInfo(components.template, filePath, content);
    promises[1] = processItem(components.template, options).then(() => compileHtml(components.template, options));
  }

  promises[2] = Promise.all(components.styles.map((style, index) => {
    style.index = index;
    setSourceInfo(style, filePath, content);
    return processItem(style, options);
  }));

  promises[3] = Promise.all(components.customBlocks.map((block, index) => {
    block.index = index;
    setSourceInfo(block, filePath, content);
    return processItem(block, options);
  }));

  await Promise.all(promises);

  return generate(filePath, components, options);
};

/**
 * Generate source code from given components
 *
 * @param {string}          filePath
 * @param {SFCDescriptor}   components
 * @param {CompilerOptions} options
 * @return {BuildResult} Gnerated result
 */


async function generate(filePath, components, options) {
  const rootNode = new _sourceMap.SourceNode(null, null, filePath);
  const hookNode = new _sourceMap.SourceNode(null, null, filePath);

  const scopeId = (0, _GenId2.default)(filePath);
  const extractedStyles = [];

  let warnings = [];
  let hasScopedStyles = false;

  /**
   * Process components.script
   */

  rootNode.add(['module.exports = (', options.runtime, '\n)({\n']);

  if (components.script) {
    rootNode.add('script: function(module, exports) {\n');
    if (components.script.attrs.src) {
      rootNode.add(['module.exports = (', components.script.node, '\n)\n']);
    } else {
      rootNode.add([components.script.node, '\n']);
    }
    rootNode.add('},\n');
    warnings = warnings.concat(components.script.warnings || []);
  }

  /**
   * Process components.template
   */

  if (components.template) {
    rootNode.add(['template: ', components.template.node, ',\n']);
    if (components.template.attrs.functional) {
      rootNode.add('functional: true,\n');
    }
    warnings = warnings.concat(components.template.warnings || []);
  }

  /**
   * Process components.styles
   */

  if (components.styles.length) {
    const externalStyles = [];
    const inlineStyles = [];

    let hasScopedStyles = false;
    let cssModules = null;

    for (const style of components.styles) {
      let node = style.node;

      if (style.attrs.src) {
        externalStyles.push(node);
        continue;
      }

      const postcssPlugins = options.postcss.slice();

      /**
       * Module style
       */

      if (style.attrs.module) {
        let name = '$style';
        if (style.attrs.module !== true) {
          name = style.attrs.module;
        }
        if (!cssModules) {
          cssModules = {};
        }
        postcssPlugins.push((0, _postcssPluginComposition2.default)([(0, _postcssModules2.default)(_extends({
          getJSON(fileName, mapping) {
            cssModules[name] = mapping;
          }
        }, options.postcssModules))]));
      }

      /**
       * Scoped style
       */

      if (style.scoped) {
        hasScopedStyles = true;
        postcssPlugins.push((0, _PostCSSScope2.default)({ id: scopeId }));
      }

      /**
       * PostCSS transform
       */

      if (postcssPlugins.length) {
        let postcssMapOpts = false;

        if (options.styleSourceMap) {
          postcssMapOpts = { inline: false, annotation: false, prev: node.toStringWithSourceMap().map.toJSON() };
        }

        const result = await (0, _postcss2.default)(postcssPlugins).process(node.toString(), { map: postcssMapOpts, from: style.filePath, to: style.filePath });

        if (!options.styleSourceMap) {
          node = new _sourceMap.SourceNode(null, null, node.source, result.css);
        } else {
          node = _sourceMap.SourceNode.fromStringWithSourceMap(result.css, new _sourceMap.SourceMapConsumer(result.map.toJSON()));
        }
      }

      /**
       * Style output
       */

      if (!options.extractStyles) {
        inlineStyles.push(node);
      } else if (options.styleSourceMap) {
        extractedStyles.push(node.toStringWithSourceMap({ sourceRoot: options.sourceMapRoot }));
      } else {
        extractedStyles.push({ code: node.toString() });
      }

      warnings = warnings.concat(style.warnings || []);
    }

    /**
     * Bundle styles
     */

    if (hasScopedStyles) {
      rootNode.add('hasScopedStyles: true,\n');
    }

    if (externalStyles.length || inlineStyles.length) {
      rootNode.add('inlineStyles: [\n');
      for (const externalNode of externalStyles) {
        rootNode.add(['(', externalNode, '\n),\n']);
      }
      if (inlineStyles.length) {
        const inlineNode = new _sourceMap.SourceNode(null, null, filePath, inlineStyles);
        if (options.styleSourceMap) {
          const result = inlineNode.toStringWithSourceMap({ sourceRoot: options.sourceMapRoot });
          rootNode.add(JSON.stringify(result.code + `/*# sourceMappingURL=data:application/json;base64,${new Buffer(result.map.toString()).toString('base64')} */`));
        } else {
          rootNode.add(JSON.stringify(inlineNode.toString()));
        }
        rootNode.add(',\n');
      }
      rootNode.add('],\n');
    }

    if (cssModules) {
      rootNode.add(['cssModules: ', JSON.stringify(cssModules), ',\n']);
    }
  }

  /**
   * Custom blocks
   */

  if (components.customBlocks.length) {
    rootNode.add('customBlocks: [\n');
    for (const block of components.customBlocks) {
      rootNode.add('function(module, exports) {\n');
      if (block.attrs.src) {
        rootNode.add(['module.exports = (', block.node, '\n)\n']);
      } else {
        rootNode.add([block.node, '\n']);
      }
      rootNode.add('},\n');
      warnings = warnings.concat(block.warnings || []);
    }
    rootNode.add('],\n');
  }

  if (options.serverRendering) {
    rootNode.add('server: true,\n');
  }

  if (options.hotReload) {
    rootNode.add('hotAPI: module.hot,\n');
  }

  if (options.includeFileName) {
    rootNode.add(['file: ', JSON.stringify(filePath), ',\n']);
  }

  rootNode.add(['scopeId: ', JSON.stringify(scopeId), ',\n', '})\n']);

  /**
   * Return result
   */

  let result = null;

  if (options.sourceMap) {
    result = rootNode.toStringWithSourceMap({ sourceRoot: options.sourceMapRoot });
  } else {
    result = { code: rootNode.toString() };
  }

  result.warnings = warnings;
  result.scopeId = scopeId;

  if (options.extractStyles) {
    result.extractedStyles = extractedStyles;
  }

  return result;
}

/**
 * Process the SFCBlock according to its language
 *
 * @param {SFCBlock}        item    The SFCBlock to be compiled
 * @param {CompilerOptions} options
 */
async function processItem(item, options) {
  let compile = await options.getCompiler(item, options);

  if (!compile && compile !== false) {
    compile = options.compilers[item.lang] || options.compilers[item.type];
  }

  if (!compile) {
    return;
  }

  const result = await compile(item, options);

  if (!result) {
    return;
  }

  item.warnings = result.warnings || [];

  if (!options.sourceMap || !result.map) {
    item.node = new _sourceMap.SourceNode(null, null, item.filePath, result.code);
  } else {
    item.node = _sourceMap.SourceNode.fromStringWithSourceMap(result.code, new _sourceMap.SourceMapConsumer(result.map));
  }

  item.node.setSourceContent(item.filePath, item.sourceContent);
}

/**
 * Compile html to vue template functions
 *
 * @param {SFCBlock}        item    The template block
 * @param {CompilerOptions} options
 */
function compileHtml(item, options) {

  if (item.attrs.src) {
    return;
  }

  const method = options.serverRendering ? 'ssrCompile' : 'compile';
  const result = VueCompiler.compile(item.node.toString(), options.compilerOptions);
  const fnArgs = item.attrs.functional ? '_h,_vm' : '';

  let code = `({ render: function(${fnArgs}) { ${result.render} }, staticRenderFns: [ `;

  for (const fn of result.staticRenderFns) {
    code += `function(${fnArgs}) { ${fn} }, `;
  }

  code += '] })';

  code = (0, _buble.transform)(code, { transforms: { stripWith: true, stripWithFunctional: item.attrs.functional } }).code;

  item.warnings = item.warnings.concat(result.errors).concat(result.tips);
  item.node = new _sourceMap.SourceNode(null, null, item.node.source, code);
}

/**
 * Set source information for block
 *
 * @param {SFCBlock} item
 * @param {string}   filePath
 * @param {string}   content
 */
function setSourceInfo(item, filePath, content) {
  const lines = item.content.split('\n');

  item.warnings = [];

  item.line = content.slice(0, item.start).split(/\r?\n/g).length;

  item.sourceContent = content;
  item.content = Array(item.line).join('\n') + item.content;

  item.filePath = `${filePath}?${item.type}`;
  if (item.index != null) {
    item.filePath += `_${item.index}`;
  }

  if (item.attrs.src) {
    item.node = new _sourceMap.SourceNode(item.line || 1, item.column, item.filePath, `require(${JSON.stringify(item.attrs.src)})`);
    return;
  }

  item.node = new _sourceMap.SourceNode(null, null, filePath, lines.map((content, line) => {
    let lineEnding = '';

    if (line + 1 < lines.length) {
      lineEnding = '\n';
    }

    return new _sourceMap.SourceNode((item.line || 1) + line, line ? 0 : item.column, filePath, content + lineEnding);
  }));
}